/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* ControlWidget.cpp
   Gabriel M. Beddingfield
   2008-06-08


*/


#include <QtGui>
#include <QDebug>
#include <memory>
#include <iostream>

#include "TimeBase.hpp"
#include "TimeBaseController.hpp"
#include "ControlWidget.hpp"

using namespace InConcert;

ControlWidget::ControlWidget(QWidget* parent, TimeBaseController* tb) :
    QWidget(parent),
    m_tb(tb)
{
    m_layout = new QGridLayout(this);

    m_tempo_lbl = new QLabel(this);
    m_tempo_lbl->setText("120.0");
    int row = 0;
    int col = 0;
    m_layout->addWidget(m_tempo_lbl, row, col);
    col++;

    m_tap_btn = new QPushButton(this);
    m_tap_btn->setIcon(QIcon(":tap1.png"));
    m_tap_btn->setToolTip(tr("Tap"));
    connect(m_tap_btn, SIGNAL(clicked()),
            m_tb, SLOT(tap()));
    m_layout->addWidget(m_tap_btn, row, col);
    col++;

    m_cue = new QPushButton(this);
    m_cue->setIcon(QIcon(":cue1.png"));
    m_cue->setToolTip(tr("Cue"));
    m_cue->setCheckable(true);
    m_cue->resize(m_cue->minimumSize());
    connect(m_cue, SIGNAL(toggled(bool)),
            m_tb, SLOT(setAutoStart(bool)));
    m_layout->addWidget(m_cue, row, col);
    col++;

    m_re_zero = new QPushButton(this);
    m_re_zero->setIcon(QIcon(":rewind1.png"));
    m_re_zero->setToolTip(tr("Re-zero transport"));
    connect(m_re_zero, SIGNAL(clicked()),
            m_tb, SLOT(re_zero()));
    m_layout->addWidget(m_re_zero, row, col);

    row++;
    col = 0;

    m_play = new QPushButton(this);
    m_play->setIcon(QIcon(":play1.png"));
    m_play->setToolTip(tr("Start Transport"));
    connect(m_play, SIGNAL(clicked()),
            m_tb, SLOT(play()));
    m_layout->addWidget(m_play, row, col);
    col++;

    m_stop = new QPushButton(this);
    m_stop->setIcon(QIcon(":pause1.png"));
    m_stop->setToolTip(tr("Stop Transport"));
    connect(m_stop, SIGNAL(clicked()),
            m_tb, SLOT(stop()));
    m_layout->addWidget(m_stop, row, col);
    col++;

    m_insert_beat = new QPushButton(this);
    m_insert_beat->setIcon(QIcon(":backward1.png"));
    m_insert_beat->setToolTip(tr("Insert a beat"));
    connect(m_insert_beat, SIGNAL(clicked()),
            m_tb, SLOT(insert_beat()));
    m_layout->addWidget(m_insert_beat, row, col);
    col++;

    m_skip_beat = new QPushButton(this);
    m_skip_beat->setIcon(QIcon(":forward1.png"));
    m_skip_beat->setToolTip(tr("Skip a beat"));
    connect(m_skip_beat, SIGNAL(clicked()),
            m_tb, SLOT(skip_beat()));
    m_layout->addWidget(m_skip_beat, row, col);
    col++;

    m_show_settings = new QPushButton(this);
    m_show_settings->setIcon(QIcon(":setup1.png"));
    m_show_settings->setToolTip(tr("Show Settings"));
    connect(m_show_settings, SIGNAL(clicked()),
            parent, SLOT(switch_widget()));
    m_layout->addWidget(m_show_settings, row, col);
    col++;
}
