/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Performances           *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef INCONCERT_EPOCH_HPP__
#define INCONCERT_EPOCH_HPP__

/* Epoch.hpp
   Gabriel M. Beddingfield
   2008-01-29

*/

#include <jack/types.h>
#include <ostream>
#include <QReadWriteLock>

namespace InConcert
{
/**
 * Convencience struct for TimeLine implementations.
 *
 * This struct represents a reference point from which time is
 * measured, and offers some BBT calculation services.
 */
struct Epoch
{
    jack_nframes_t m_frame;         //< audio frame (sample) from beginning of song
    int32_t m_bar,                  //< bar (measure) from beginning of song [1, inf)
        m_beat,                     //< beat within bar [1, m_beats_per_bar]
        m_tick,                     //< tick within beat [0, m_ticks_per_beat - 1]
        m_beats_per_bar,            //< the time signature [1, inf)
        m_ticks_per_beat;           //< tick rate (should be constant) [1, inf)
    double m_beats_per_minute;      //< the tempo [1, inf)
    // Metadata to speed computation:
    double m_frames_per_beat,       //< Number of audio frames (samples) in a beat
        m_frames_per_tick,          //< Number of audio frames (samples) in a tick
        m_frames_per_measure,       //< Number of audio frames (samples) in a bar
        m_anchor_beat_abs;          //< Approx. absolue beat in song (first beat is 0.0)
    mutable QReadWriteLock m_mutex; //< Mutex that protects all internal variables

    Epoch(void);
    ~Epoch();
    Epoch(const Epoch& o);
    Epoch& operator=(const Epoch& o);

    void init_frame_rates(jack_nframes_t rate);
    void set_jack_position_meta(jack_position_t* pos);

    void getBBT(jack_nframes_t fr,
                int32_t* bar,
                int32_t* beat,
                int32_t* tick,
                jack_nframes_t* frames) const;
    jack_nframes_t getFrame(double bar,
                            double beat,
                            double tick,
                            double frames = 0.0) const;
}; // class Epoch

} /* namespace InConcert */

std::ostream& operator<<(std::ostream& os, InConcert::Epoch x);

#endif // INCONCERT_EPOCH_HPP__
