/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HIJACK_CLIENT_HPP__
#define HIJACK_CLIENT_HPP__

/* Client.hpp
 *
 * This class manages the applications interface to a Jack server.
 */

#include <string>
#include <set>
#include <QMutex>
#include <jack/types.h>
#include <jack/transport.h>

namespace HiJack
{
class Port;
class TimebaseMaster;

class Client
{
public:
    Client(const std::string& name);
    virtual ~Client(void);

    const std::string& getName(void) const;
    bool registerPort(Port* port);
    bool unregisterPort(Port* port);
    bool addPortToProcessCallback(Port* port);
    bool removePortFromProcessCallback(Port* port);
    bool registerTimebaseMaster(TimebaseMaster* master, bool if_none_already = false);
    bool unregisterTimebaseMaster(TimebaseMaster* tbm = 0);
    TimebaseMaster* getTimebaseMaster(void) const;


    //std::list<std::string> getMidiOutputPortList(void);

    jack_client_t* ref(void);

private:
    // The following should only be called when the mutex has
    // been locked.
    void open_locked(void);
    void close_locked(void);
    void activate_locked(void);
    void deactivate_locked(void);
    bool removePortFromProcessCallback_locked(Port *port);

    static int jackProcessCallback(jack_nframes_t nframes,
                                   void* arg);
    static void jackTimebaseCallback(jack_transport_state_t state,
                                     jack_nframes_t nframes,
                                     jack_position_t *pos,
                                     int new_pos,
                                     void* arg);

    // Sorts ports for use in the jackProcessCallback.
    struct procSort {
        bool operator()(Port* left, Port* right);
    };

    typedef std::set<Port*, procSort> port_list_t;

    jack_client_t* m_client;
    std::string m_name;
    port_list_t m_ports;
    port_list_t m_process;  // ports in the callback cycle
    TimebaseMaster *m_timebase;

    mutable QMutex m_mutex;
};

} // namespace HiJack

#endif // HIJACK_CLIENT_HPP__
