/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HIJACK_MIDIINPORT_HPP__
#define HIJACK_MIDIINPORT_HPP__

/* MidiInPort.hpp
 *
 */

#ifdef JACK_MIDI_SUPPORT

#include <jack/jack.h>
#include <jack/midiport.h>
#include <list>
#include <QtCore/QString>
#include <memory>
#include "Port.hpp"

namespace HiJack
{

class MidiInPort : public Port
{
public:
    MidiInPort(const std::string& name);
    MidiInPort(Client *client,
               const std::string& name);
    virtual ~MidiInPort();

    // Ref. http://www.midi.org/about-midi/table1.shtml
    typedef uint8_t byte;
    virtual void handleNoteOn(byte chan,
                              byte note,
                              byte velocity,
                              jack_nframes_t frame);
    virtual void handleNoteOff(byte chan,
                               byte note,
                               byte velocity,
                               jack_nframes_t frame);
    virtual void handlePolyphonicKeyPressure(
        byte chan,
        byte note,
        byte value,
        jack_nframes_t frame);
    virtual void handleControlChange(
        byte chan,
        byte ctrl_no,
        byte value,
        jack_nframes_t frame);
    virtual void handleProgramChange(
        byte chan,
        byte program,
        jack_nframes_t frame);
    virtual void handleChannelPressure(
        byte chan,
        byte value,
        jack_nframes_t frame);
    virtual void handlePitchWheel(byte chan,
                                  uint16_t value,
                                  jack_nframes_t frame);
    // System Common
    virtual void handleSysCommExclusive(
        byte vendid,
        const std::list<byte>& data,
        jack_nframes_t frame);
    virtual void handleSysCommMTCQuarterFrame(
        byte value,
        jack_nframes_t frame);
    virtual void handleSysCommSongPosition(
        uint16_t value,
        jack_nframes_t frame);
    virtual void handleSysCommSongSelect(
        byte song,
        jack_nframes_t frame);
    virtual void handleSysCommTuneRequest(jack_nframes_t frame);
    // System Realtime
    virtual void handleSysRealTimingClock(jack_nframes_t frame);
    virtual void handleSysRealStart(jack_nframes_t frame);
    virtual void handleSysRealContinue(jack_nframes_t frame);
    virtual void handleSysRealStop(jack_nframes_t frame);
    virtual void handleSysRealActiveSensing(jack_nframes_t frame);
    virtual void handleSysRealReset(jack_nframes_t frame);

protected:
    virtual int processCallback(jack_nframes_t nframes);
    void handleMidi(const jack_midi_event_t& event);

}; // MidiInPort

} // namespace HiJack

#endif // JACK_MIDI_SUPPORT

#endif // HIJACK_MIDIINPORT_HPP__
