/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Port.cpp
 * 2008-08-19
 */

#include <iostream>
#include "Client.hpp"
#include "Port.hpp"
#include <jack/jack.h>

using namespace std;
using namespace HiJack;

Port::Port(const std::string& name, PortType type) :
    m_client(0),
    m_port(0),
    m_type(type),
    m_name(name)
{
}

Port::Port(Client *client,
           const std::string& name,
           PortType type,
           JackPortFlags flags) :
    m_client(client),
    m_port(0),
    m_name(name),
    m_type(type),
    m_flags(flags)
{
    registerPort(m_client);
}

Port::~Port()
{
    unregisterPort();
}

bool Port::registerPort(Client* client)
{
    QMutexLocker mx(&m_mutex);
    bool rv = client->registerPort(this);
    return rv;
}

bool Port::unregisterPort(void)
{
    bool rv = false;
    QMutexLocker mx(&m_mutex);
    if (m_client) {
        bool rv = m_client->unregisterPort(this);
        m_client = 0;
    }
    return rv;
}

int Port::getFlags(void)
{
    return m_flags;
}

Port::PortType Port::getType(void)
{
    return m_type;
}

jack_port_t* Port::ref(void)
{
    return m_port;
}
