/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HIJACK_PORT_HPP__
#define HIJACK_PORT_HPP__

/* Port.hpp
 * 2008-08-18
 */

#include <jack/types.h>
#include <QMutex>

namespace HiJack
{
class Client;

class Port
{
public:
    friend class Client;

    typedef enum { Audio, Midi } PortType;

    Port(const std::string& name, PortType type);
    Port(Client *client,
         const std::string& name,
         PortType type,
         JackPortFlags flags);
    virtual ~Port();

    bool registerPort(Client* client);
    bool unregisterPort(void);
    int getFlags(void);
    PortType getType(void);
    const std::string& getName(void) const { return m_name; }
    jack_port_t* ref(void);

protected:
    virtual int processCallback(jack_nframes_t /* nframes */)
    { return 0; }
    virtual void jackIsClosing(void) throw()
    { }

private:
    Client *m_client;
    jack_port_t *m_port;
    std::string m_name;
    PortType m_type;
    JackPortFlags m_flags;
    mutable QMutex m_mutex;
}; // class Port

} // namespace HiJack
#endif // HIJACK_PORT_HPP__
