/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* JackMidiIn.cpp
   Gabriel M. Beddingfield
   2008-08-20

*/

#include <iostream>
#include "JackMidiIn.hpp"
#include "MidiImplementation.hpp"
#include "TimeBase.hpp"

using namespace InConcert;

JackMidiIn::~JackMidiIn()
{
}

void JackMidiIn::handleControlChange(byte chan,
                                     byte ctrl_no,
                                     byte value,
                                     jack_nframes_t frame)
{
    if ((ctrl_no == CC_SET_CHANNEL) && (value < 16)) {
        m_chan = value;
    }
    if ( (chan != m_chan)
         || (ctrl_no < MIDI_CC_MIN)
         || (ctrl_no > MIDI_CC_MAX) ) {
        return;
    }
    switch (ctrl_no) {
    case CC_REZERO:
        m_tb->re_zero();
        break;
    case CC_TAP:
        if(value & 0x40) m_tb->tap(frame);
        break;
    case CC_STARTSTOP:
        if(value & 0x40) {
            m_tb->play();
        } else {
            m_tb->stop();
        }
        break;
    case CC_INSERTORSKIP:
        if(value & 0x40) {
            m_tb->skip_beat();
        } else {
            m_tb->insert_beat();
        }
        break;
    case CC_TAPSREQUIRED:
        if((value >= 2) && (value <= TAPS_MAX)) {
            m_tb->setTapsRequired(value);
        }
        break;
    case CC_TAPSCUECTL:
        m_tb->setAutoStart(value & 0x40);
        break;
    case CC_BEATSPERMEAS:
        m_tb->setBeatsPerMeasure(value);
        break;
    case CC_SETTEMPO:
        m_tb->setTempo(25.5 + double(value)*1.50);
        break;
    default:
        break;
    }
}
