/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* SettingsWidget.cpp
   Gabriel M. Beddingfield
   2008-06-08


*/

#include <QtGui>
#include <memory>
#include "SettingsWidget.hpp"
#include "TimeBaseController.hpp"

using namespace InConcert;

SettingsWidget::SettingsWidget(QWidget* parent, TimeBaseController* tb) :
    QWidget(parent),
    m_timebase(tb)
{
    m_taps_required = new QSpinBox(this);
    m_taps_required_label = new QLabel(this);
    m_taps_required_label->setText(tr("Taps required:"));
    m_taps_required->setMinimum(2);
    m_taps_required->setMaximum(20);
    m_taps_required->setValue(m_timebase->getTapsRequired());
    connect(m_taps_required, SIGNAL(valueChanged(int)),
            m_timebase, SLOT(setTapsRequired(int)));
    connect(m_timebase, SIGNAL(tapsRequiredChanged(int)),
            m_taps_required, SLOT(setValue(int)));

    m_beats_per_measure = new QSpinBox(this);
    m_beats_per_measure_label = new QLabel(this);
    m_beats_per_measure_label->setText(tr("Beats/Measure:"));
    m_beats_per_measure->setMinimum(2);
    m_beats_per_measure->setMaximum(160);
    m_beats_per_measure->setValue(m_timebase->getBeatsPerMeasure());
    connect(m_beats_per_measure, SIGNAL(valueChanged(int)),
            m_timebase, SLOT(setBeatsPerMeasure(int)));
    connect(m_timebase, SIGNAL(beatsPerMeasureChanged(int)),
            m_beats_per_measure, SLOT(setValue(int)));

    m_taps_timeout = new QDoubleSpinBox(this);
    m_taps_timeout_label = new QLabel(this);
    m_taps_timeout_label->setText(tr("Timeout (sec):"));
    m_taps_timeout->setMinimum(0.25);
    m_taps_timeout->setMaximum(20.00);
    m_taps_timeout->setSingleStep(0.05);
    m_taps_timeout->setValue(m_timebase->getTapsTimeout());
    connect(m_taps_timeout, SIGNAL(valueChanged(double)),
            m_timebase, SLOT(setTapsTimeout(double)));
    connect(m_timebase, SIGNAL(tapsTimeoutChanged(double)),
            m_taps_timeout, SLOT(setValue(double)));


    m_show_controls = new QPushButton(this);
    m_show_controls->setText(tr("x"));
    connect(m_show_controls, SIGNAL(clicked()),
            parent, SLOT(switch_widget()));

    QHBoxLayout* hlay = new QHBoxLayout(this);
    QGridLayout* glay0 = new QGridLayout(this);
    glay0->addWidget(m_taps_required_label, 0, 0);
    glay0->addWidget(m_taps_required, 0, 1);
    glay0->addWidget(m_beats_per_measure_label, 1, 0);
    glay0->addWidget(m_beats_per_measure, 1, 1);

    hlay->addLayout(glay0);

    QHBoxLayout* hlay1 = new QHBoxLayout(this);
    QVBoxLayout* vlay1 = new QVBoxLayout(this);
    hlay1->addWidget(m_taps_timeout_label);
    hlay1->addWidget(m_taps_timeout);
    vlay1->addLayout(hlay1);
    vlay1->addStretch(0);

    hlay->addLayout(vlay1);

    QVBoxLayout* vlay2 = new QVBoxLayout(this);
    vlay2->addStretch(0);
    vlay2->addWidget(m_show_controls);

    hlay->addLayout(vlay2);
    setLayout(hlay);
}

