#ifndef INCONCERT_TAPTEMPOMEDIAN_HPP__              /*-*- indent-tabs-mode:nil;-*-*/
#define INCONCERT_TAPTEMPOMEDIAN_HPP__
/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* TapTempoMedian.hpp
   Gabriel M. Beddingfield
   2008-08-16

   Tavg = sum(Ti)/n
   Tn+1 = Tn + (Tn-Tavg)/((n-1)/2) = Tn + 2*(Tn-Tavg)/(n-1)
 */

#include "TapTempoBaseImpl.hpp"

namespace InConcert
{
template <typename ScalarT, typename TempoT>
class TapTempoMedian : public TapTempoBaseImpl<ScalarT, TempoT>
{
    ScalarT m_sum;
    ScalarT m_tap_epoch;
    ScalarT m_last_tap;
    unsigned m_taps_received;
    typedef TapTempoBaseImpl<ScalarT, TempoT> base_class;

public:
    TapTempoMedian() :
        base_class("TapTempoMedian"),
        m_sum(0),
        m_tap_epoch(0),
        m_last_tap(0),
        m_taps_received(0)
    { }

    virtual ~TapTempoMedian() { }

    virtual void invalidate(void)
    {
        m_sum = 0;
        m_tap_epoch = 0;
        m_taps_received = 0;
        base_class::invalidate();
    }

    virtual ScalarT tap(ScalarT tap)
    {
        if (this->reset() || (m_taps_received == this->getTapsRequired())) {
            invalidate();
        } else if (this->getTimeout()
                   && (tap - m_last_tap) >= this->getTimeout()) {
            invalidate();
        }

        ScalarT rv = 0;
        if (m_taps_received == 0) {
            m_tap_epoch = tap;
        }

        ScalarT dt = tap - m_tap_epoch;
        m_sum += dt;
        m_last_tap = tap;
        ++m_taps_received;
        if(m_taps_received == this->getTapsRequired()) {
            ScalarT intervals = m_taps_received - 1;
            /* 2 * (dt - average) / intervals
             * expanded out to avoid losing precision
             */
            this->setTempoByInterval( ((2*dt) - (2*m_sum/m_taps_received))/intervals );
            rv = tap + this->getTempoInterval();
            this->queueReset();
        }
        return rv;
    }

}; // class TapTempoMedian

} // namespace InConcert
#endif // INCONCERT_TAPTEMPOMEDIAN_HPP__
