/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* TimeBaseController.cpp
   Gabriel M. Beddingfield
   2008-02-05

*/

#include <QtCore>
#include <QDebug>
#include <cmath>
#include <cassert>
#include "TimeBaseController.hpp"
#include "TimeBase.hpp"

using namespace InConcert;

TimeBaseController::TimeBaseController(QObject* parent, TimeBase* tb) :
    QObject(parent),
    m_tb(0),
    m_own_tb(false)
{
    if(tb) {
        m_tb = tb;
        m_own_tb = false;
    } else {
        m_tb = new TimeBase;
        m_own_tb = true;
    }
}

TimeBaseController::~TimeBaseController()
{
    if(m_own_tb) delete m_tb;
}

int TimeBaseController::getTapsRequired(void)
{
    int tr = (int)m_tb->getTapsRequired(); // returns unsigned
    assert(tr > 2);
    return tr;
}

double TimeBaseController::getTapsTimeout(void)
{
    return m_tb->getTapsTimeout();
}

bool TimeBaseController::getAutoStart(void)
{
    return m_tb->getAutoStart();
}

int TimeBaseController::getBeatsPerMeasure(void)
{
    int bpm = (int)m_tb->getBeatsPerMeasure(); // returns unsigned
    assert(bpm > 1);
    return bpm;
}

double TimeBaseController::getTempo(void)
{
    return m_tb->getTempo();
}

void TimeBaseController::setTapsRequired(int ct)
{
    if(((unsigned)ct != m_tb->getTapsRequired()) && (ct > 1)) {
        m_tb->setTapsRequired((unsigned)ct);
        emit tapsRequiredChanged(ct);
    }
}

void TimeBaseController::setTapsTimeout(double to)
{
    if(fabs(m_tb->getTapsTimeout() - to) < .001) {
        m_tb->setTapsTimeout(to);
        emit tapsTimeoutChanged(to);
    }
}

void TimeBaseController::tap(void)
{
#warning "How do we detect when the tempo changes?"
    m_tb->tap();
    emit tapped();
}

void TimeBaseController::skip_beat(void)
{
    m_tb->skip_beat();
}

void TimeBaseController::insert_beat(void)
{
    m_tb->insert_beat();
}

void TimeBaseController::re_zero(void)
{
    m_tb->re_zero();
}

void TimeBaseController::play(void)
{
    m_tb->play();
}

void TimeBaseController::stop(void)
{
    m_tb->stop();
}

void TimeBaseController::setAutoStart(bool tf)
{
    if(tf != m_tb->getAutoStart()) {
        m_tb->setAutoStart(tf);
        emit autoStartChanged(tf);
    }
}

void TimeBaseController::setBeatsPerMeasure(int b)
{
    if(((unsigned)b != m_tb->getBeatsPerMeasure()) && (b > 1)) {
        m_tb->setBeatsPerMeasure((unsigned)b);
        emit beatsPerMeasureChanged(b);
    }
}

void TimeBaseController::setTempo(double t)
{
    if(fabs(t - m_tb->getTempo()) < .001) {
        m_tb->setTempo(t);
        emit tempoChanged(t);
    }
}


