/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Performances           *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef INCONCERT_TIMEBASECONTROLLER_HPP__
#define INCONCERT_TIMEBASECONTROLLER_HPP__

/* TimeBaseController.hpp
   Gabriel M. Beddingfield
   2008-02-05

   Qt signal/slot wrapper for the TimeBase class.  See TimeBase class
   to understand what each method is for.
*/

#include <QObject>

namespace InConcert
{
class TimeBase;

class TimeBaseController : public QObject
{
    Q_OBJECT
public:
    TimeBaseController(QObject* parent = 0, TimeBase* tb = 0);
    ~TimeBaseController();
    int getTapsRequired(void);
    double getTapsTimeout(void);
    bool getAutoStart(void);
    int getBeatsPerMeasure(void);
    double getTempo(void);

public slots:
    void setTapsRequired(int);
    void setTapsTimeout(double);
    void tap(void);
    void skip_beat(void);
    void insert_beat(void);
    void re_zero(void);
    void play(void);
    void stop(void);
    void setAutoStart(bool);
    void setBeatsPerMeasure(int);
    void setTempo(double);

signals:
    void tapsRequiredChanged(int);
    void tapsTimeoutChanged(double);
    void autoStartChanged(bool);
    void beatsPerMeasureChanged(int);
    void tempoChanged(double);
    void tapped(void);

private:
    TimeBase* m_tb;
    bool m_own_tb;
};
}

#endif // INCONCERT_TIMEBASECONTROLLER_HPP__
