/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* main.cpp
   Gabriel M. Beddingfield
   2008-01-29

   To understand the code, see Documentation/design.txt.
*/

// Standard C++ libraries
#include <iostream>
#include <memory>
#include "HiJack/Client.hpp"
#include "TimeBase.hpp"
#include "JackMidiIn.hpp"
#include "AlsaMidiIn.hpp"
#include "InConcertDlg.hpp"
#include "global.hpp"
#include <QApplication>

using namespace std;
using namespace InConcert;

// prototype declared in global.hpp
namespace InConcert
{

HiJack::Client& getJackClientInstance(void)
{
    static auto_ptr<HiJack::Client> d;
    if (d.get() == 0) {
        d.reset( new HiJack::Client("InConcert") );
    }
    return *d;
}

} /* namespace InConcert */

int main(int argc, char* argv[])
{
    int rv;
    QApplication app(argc, argv);
    HiJack::Client& jack = getJackClientInstance();
    TimeBase tb;
    JackMidiIn j_midi(&tb);

    jack.registerTimebaseMaster(&tb);
    jack.registerPort(&j_midi);
    jack.addPortToProcessCallback(&j_midi);

    AlsaMidiIn ml(&tb);
    InConcertDlg c(0, &tb);

    ml.start();
    c.show();
    rv = app.exec();
    ml.terminate();

    return rv;
}

